@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: fnt.h,v 1.4 2001/04/09 00:24:07 deca Exp $
+     $Id: fnt.h,v 1.15 2004/02/15 19:51:23 stromberg Exp $
 */
 
 
@@ -27,6 +27,13 @@
 
 #include "sg.h"
 
+#ifdef UL_MAC_OSX
+#  include <OpenGL/gl.h>
+#else
+#  include <GL/gl.h>
+#endif
+
+
 #define FNTMAX_CHAR  256
 #define FNT_TRUE  1
 #define FNT_FALSE 0
@@ -59,7 +66,7 @@ public:
   virtual float getWidth     () const    = 0 ;
   virtual float getGap       () const    = 0 ;
 
-  virtual int hasGlyph ( char c ) = 0 ;
+  virtual int hasGlyph ( char c ) const = 0 ;
 } ;
 
 
@@ -90,6 +97,10 @@ private:
     v_bot==0.0 and v_top==1.0.
   */
 
+  /* Nominal baseline widths */
+
+  float widths  [ FNTMAX_CHAR ] ;
+
   /* Texture coordinates */
 
   float t_top   [ FNTMAX_CHAR ] ; /* Top    edge of each character [0..1] */
@@ -171,19 +182,39 @@ public:
   float getGap       () const { return gap       ; } 
 
 
-  void setGlyph ( char c,
+  void setGlyph ( char c, float wid,
                   float tex_left, float tex_right,
                   float tex_bot , float tex_top  ,
                   float vtx_left, float vtx_right,
                   float vtx_bot , float vtx_top  ) ;
+  void setGlyph ( char c,
+                  float tex_left, float tex_right,
+                  float tex_bot , float tex_top  ,
+                  float vtx_left, float vtx_right,
+                  float vtx_bot , float vtx_top  ) /* deprecated */
+  {
+    setGlyph ( c, vtx_right,
+               tex_left, tex_right, tex_bot, tex_top,
+               vtx_left, vtx_right, vtx_bot, vtx_top ) ;
+  }
   
-  int  getGlyph ( char c,
+  int  getGlyph ( char c, float* wid,
                   float *tex_left = NULL, float *tex_right = NULL,
                   float *tex_bot  = NULL, float *tex_top   = NULL,
                   float *vtx_left = NULL, float *vtx_right = NULL,
                   float *vtx_bot  = NULL, float *vtx_top   = NULL) ;
+  int  getGlyph ( char c,
+                  float *tex_left = NULL, float *tex_right = NULL,
+                  float *tex_bot  = NULL, float *tex_top   = NULL,
+                  float *vtx_left = NULL, float *vtx_right = NULL,
+                  float *vtx_bot  = NULL, float *vtx_top   = NULL) /* deprecated */
+  {
+    return getGlyph ( c, NULL,
+                      tex_left, tex_right, tex_bot, tex_top,
+                      vtx_left, vtx_right, vtx_bot, vtx_top ) ;
+  }
 
-  int hasGlyph ( char c ) { return getGlyph ( c ) ; }
+  int hasGlyph ( char c ) const { return exists[ (GLubyte) c ] ; }
 
   void getBBox ( const char *s, float pointsize, float italic,
                  float *left, float *right,
@@ -213,6 +244,69 @@ public:
 } ;
 
 
+
+class fntBitmapFont : public fntFont
+{
+ protected:
+    
+    const GLubyte **data;
+    int first;
+    int count;
+    int height;
+    float xorig, yorig;
+    int fix;
+    float wid, gap;
+
+ public:
+
+    // data is a null-terminated list of glyph images:
+    //   data[i][0]    - image width
+    //   data[i][1..n] - packed bitmap
+
+    fntBitmapFont ( const GLubyte **data, int first, int height, 
+		    float xorig, float yorig ) ;
+
+    virtual ~fntBitmapFont () ;
+
+    virtual void getBBox ( const char *s, float pointsize, float italic,
+			   float *left, float *right,
+			   float *bot , float *top ) ;
+
+    virtual void putch ( sgVec3 curpos, float pointsize, float italic, char c ) ;
+    virtual void puts ( sgVec3 curpos, float pointsize, float italic, const char *s ) ;
+
+    virtual void begin () ;
+    virtual void end () ;
+    
+    virtual int load ( const char *fname, GLenum mag, GLenum min ) { return -1; }
+    
+    virtual void setFixedPitch ( int f )    { fix = f;    }
+    virtual int   isFixedPitch () const     { return fix; }
+    
+    virtual void  setWidth     ( float w )  { wid = w;    }
+    virtual void  setGap       ( float g )  { gap = g;    }
+    
+    virtual float getWidth     () const     { return wid; }
+    virtual float getGap       () const     { return gap; }
+    
+    virtual int hasGlyph ( char c ) const ;
+};
+
+
+/* Builtin Bitmap Fonts */
+
+#define FNT_BITMAP_8_BY_13         0
+#define FNT_BITMAP_9_BY_15         1
+#define FNT_BITMAP_HELVETICA_10    2
+#define FNT_BITMAP_HELVETICA_12    3
+#define FNT_BITMAP_HELVETICA_18    4
+#define FNT_BITMAP_TIMES_ROMAN_10  5
+#define FNT_BITMAP_TIMES_ROMAN_24  6
+
+fntBitmapFont *fntGetBitmapFont(int id);
+
+
+
 class fntRenderer
 {
   fntFont *font ;
@@ -260,5 +354,7 @@ public:
 } ;
 
 
+void fntInit () ;
+
 #endif
 
