@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,19 +13,25 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: netSocket.cxx,v 1.13 2001/06/16 21:03:27 sjbaker Exp $
+     $Id: netSocket.cxx,v 1.28 2004/03/22 19:44:50 sjbaker Exp $
 */
 
 #include "netSocket.h"
 
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
 
+#if defined(UL_MAC_OSX)
+#  include <netinet/in.h>
+#endif
+
+#include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <time.h>
 #include <sys/time.h>    /* Need both for Mandrake 8.0!! */
@@ -40,19 +46,25 @@
 
 #endif
 
-#if !defined(STDC_HEADERS) && !defined(socklen_t)
+#if defined(UL_MSVC) && !defined(socklen_t)
 #define socklen_t int
 #endif
 
-netAddress::netAddress ( cchar* host, int port )
+/* Paul Wiltsey says we need this for Solaris 2.8 */
+ 
+#ifndef INADDR_NONE
+#define INADDR_NONE ((unsigned long)-1)
+#endif
+                                                                                               
+netAddress::netAddress ( const char* host, int port )
 {
   set ( host, port ) ;
 }
 
 
-void netAddress::set ( cchar* host, int port )
+void netAddress::set ( const char* host, int port )
 {
-	memset(this, 0, sizeof(netAddress));
+  memset(this, 0, sizeof(netAddress));
 
   sin_family = AF_INET ;
   sin_port = htons (port);
@@ -62,12 +74,11 @@ void netAddress::set ( cchar* host, int port )
   ** to do the work; the names "" and "<broadcast>" are special.
   */
 
-	if (host[0] == '\0') {
-		sin_addr = INADDR_ANY;
-	}
-	if (host[0] == '<' && strcmp(host, "<broadcast>") == 0) {
-		sin_addr = INADDR_BROADCAST;
-	}
+  if (host[0] == '\0')
+    sin_addr = INADDR_ANY;
+
+  if (host[0] == '<' && strcmp(host, "<broadcast>") == 0)
+    sin_addr = INADDR_BROADCAST;
   else
   {
 #if 0
@@ -104,10 +115,10 @@ void netAddress::set ( cchar* host, int port )
    This is always a string of the form 'dd.dd.dd.dd' (with variable
    size numbers). */
 
-cchar* netAddress::getHost () const
+const char* netAddress::getHost () const
 {
 #if 0
-  cchar* buf = inet_ntoa ( sin_addr ) ;
+  const char* buf = inet_ntoa ( sin_addr ) ;
 #else
   static char buf [32];
 	long x = ntohl(sin_addr);
@@ -124,7 +135,7 @@ int netAddress::getPort() const
   return ntohs(sin_port);
 }
 
-cchar* netAddress::getLocalHost ()
+const char* netAddress::getLocalHost ()
 {
   //gethostbyname(gethostname())
 
@@ -132,16 +143,24 @@ cchar* netAddress::getLocalHost ()
   memset(buf, 0, sizeof(buf));
   gethostname(buf, sizeof(buf)-1);
   const hostent *hp = gethostbyname(buf);
-	if (hp && *hp->h_addr_list)
+
+  if (hp && *hp->h_addr_list)
   {
-    in_addr addr = *((in_addr*)*hp->h_addr_list);
-    cchar* host = inet_ntoa(addr);
+    in_addr     addr = *((in_addr*)*hp->h_addr_list);
+    const char* host = inet_ntoa(addr);
+
     if ( host )
       return host ;
-	}
+  }
+
   return "127.0.0.1" ;
 }
 
+
+bool netAddress::getBroadcast () const {
+    return sin_addr == INADDR_BROADCAST;
+}
+
 netSocket::netSocket ()
 {
   handle = -1 ;
@@ -171,7 +190,7 @@ netSocket::setBlocking ( bool blocking )
 {
   assert ( handle != -1 ) ;
 
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
 
 	int delay_flag = ::fcntl (handle, F_GETFL, 0);
 	if (blocking)
@@ -188,8 +207,29 @@ netSocket::setBlocking ( bool blocking )
 #endif
 }
 
+void
+netSocket::setBroadcast ( bool broadcast )
+{
+  assert ( handle != -1 ) ;
+  int result;
+  if ( broadcast ) {
+      int one = 1;
+#ifdef UL_WIN32
+      result = ::setsockopt( handle, SOL_SOCKET, SO_BROADCAST, (char*)&one, sizeof(one) );
+#else
+      result = ::setsockopt( handle, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one) );
+#endif
+  } else {
+      result = ::setsockopt( handle, SOL_SOCKET, SO_BROADCAST, NULL, 0 );
+  }
+  if ( result < 0 ) {
+      perror("set broadcast:");
+  }
+  assert ( result != -1 );
+}
+
 int
-netSocket::bind ( cchar* host, int port )
+netSocket::bind ( const char* host, int port )
 {
   assert ( handle != -1 ) ;
   netAddress addr ( host, port ) ;
@@ -207,15 +247,18 @@ int
 netSocket::accept ( netAddress* addr )
 {
   assert ( handle != -1 ) ;
-  socklen_t addr_len = sizeof(netAddress) ;
+  socklen_t addr_len = (socklen_t) sizeof(netAddress) ;
   return ::accept(handle,(sockaddr*)addr,&addr_len);
 }
 
 int
-netSocket::connect ( cchar* host, int port )
+netSocket::connect ( const char* host, int port )
 {
   assert ( handle != -1 ) ;
   netAddress addr ( host, port ) ;
+  if ( addr.getBroadcast() ) {
+      setBroadcast( true );
+  }
   return ::connect(handle,(const sockaddr*)&addr,sizeof(netAddress));
 }
 
@@ -223,14 +266,14 @@ int
 netSocket::send (const void * buffer, int size, int flags)
 {
   assert ( handle != -1 ) ;
-  return ::send (handle, (cchar*)buffer, size, flags);
+  return ::send (handle, (const char*)buffer, size, flags);
 }
 
 int
 netSocket::sendto ( const void * buffer, int size, int flags, const netAddress* to )
 {
   assert ( handle != -1 ) ;
-  return ::sendto(handle,(cchar*)buffer,size,flags,(const sockaddr*)to,sizeof(netAddress));
+  return ::sendto(handle,(const char*)buffer,size,flags,(const sockaddr*)to,sizeof(netAddress));
 }
 
 int
@@ -244,7 +287,7 @@ int
 netSocket::recvfrom ( void * buffer, int size, int flags, netAddress* from )
 {
   assert ( handle != -1 ) ;
-  socklen_t fromlen = sizeof(netAddress) ;
+  socklen_t fromlen = (socklen_t) sizeof(netAddress) ;
   return ::recvfrom(handle,(char*)buffer,size,flags,(sockaddr*)from,&fromlen);
 }
 
@@ -253,7 +296,7 @@ netSocket::close (void)
 {
   if ( handle != -1 )
   {
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
     ::close( handle );
 #else
     ::closesocket( handle );
@@ -265,7 +308,7 @@ netSocket::close (void)
 bool
 netSocket::isNonBlockingError ()
 {
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
   switch (errno) {
   case EWOULDBLOCK: // always == NET_EAGAIN?
   case EALREADY:
@@ -278,7 +321,7 @@ netSocket::isNonBlockingError ()
   if ( wsa_errno != 0 )
   {
     WSASetLastError(0);
-    fprintf(stderr,"WSAGetLastError() => %d\n",wsa_errno);
+    ulSetError(UL_WARNING,"WSAGetLastError() => %d",wsa_errno);
     switch (wsa_errno) {
     case WSAEWOULDBLOCK: // always == NET_EAGAIN?
     case WSAEALREADY:
@@ -364,7 +407,7 @@ netSocket::select ( netSocket** reads, netSocket** writes, int timeout )
 /* Init/Exit functions */
 static void netExit ( void )
 {
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
 #else
 	/* Clean up windows networking */
 	if ( WSACleanup() == SOCKET_ERROR ) {
@@ -377,17 +420,24 @@ static void netExit ( void )
 }
 
 int netInit ( int* argc, char** argv )
+{
+  /* Legacy */
+
+  return netInit () ;
+}
+
+int netInit ()
 {
   assert ( sizeof(sockaddr_in) == sizeof(netAddress) ) ;
 
-#if defined(__CYGWIN__) || !defined (WIN32)
+#if defined(UL_CYGWIN) || !defined (UL_WIN32)
 #else
 	/* Start up the windows networking */
 	WORD version_wanted = MAKEWORD(1,1);
 	WSADATA wsaData;
 
 	if ( WSAStartup(version_wanted, &wsaData) != 0 ) {
-		fprintf(stderr,"Couldn't initialize Winsock 1.1\n");
+		ulSetError(UL_WARNING,"Couldn't initialize Winsock 1.1");
 		return(-1);
 	}
 #endif
@@ -396,7 +446,7 @@ int netInit ( int* argc, char** argv )
 	return(0);
 }
 
-cchar* netFormat ( cchar* format, ... )
+const char* netFormat ( const char* format, ... )
 {
   static char buffer[ 256 ];
   va_list argptr;
@@ -405,3 +455,5 @@ cchar* netFormat ( cchar* format, ... )
   va_end(argptr);
   return( buffer );
 }
+
+
