@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: slDSP.cxx,v 1.9 2001/05/18 04:05:00 sjbaker Exp $
+     $Id: slDSP.cxx,v 1.22 2003/07/04 14:33:03 puggles Exp $
 */
 
 
@@ -33,7 +33,7 @@ static int init_bytes ;
 /* OSSAUDIO - Linux, FreeBSD, etc                               */
 /* ------------------------------------------------------------ */
 
-void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
+void slDSP::open ( const char *device, int _rate, int _stereo, int _bps )
 {
   fd = ::open ( device, O_WRONLY | O_NONBLOCK ) ;
 
@@ -155,7 +155,7 @@ void slDSP::stop ()
 
 #endif
 
-#ifdef WIN32
+#ifdef UL_WIN32
 
 /* ------------------------------------------------------------ */
 /* win32                                                        */
@@ -191,7 +191,7 @@ void CALLBACK waveOutProc( HWAVEOUT hwo, UINT uMsg,
 }
 
 
-void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
+void slDSP::open ( const char *device, int _rate, int _stereo, int _bps )
 {
    MMRESULT	result;
    int i;
@@ -357,7 +357,7 @@ float slDSP::secondsRemaining ()
    if ( error )
       return 0.0f ;
    
-   return 0.0f ;
+   return 10.0f ;
 }
 
 
@@ -413,8 +413,8 @@ void slDSP::stop ()
 /* NetBSD/OpenBSD 2.3 this should be very close to SUN Audio    */
 /* ------------------------------------------------------------ */
 
-#elif defined(__NetBSD__) || defined(__OpenBSD__) || defined(SOLARIS)
-void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
+#elif (defined(UL_BSD) && !defined(__FreeBSD__)) || defined(UL_SOLARIS)
+void slDSP::open ( const char *device, int _rate, int _stereo, int _bps )
 {
 
   counter = 0;
@@ -440,13 +440,13 @@ void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
       return;
     }
       
-#ifdef SOLARIS
+#ifdef UL_SOLARIS
 	AUDIO_INITINFO(&ainfo);
 #endif
 
     ainfo.play.sample_rate  = _rate;
     ainfo.play.precision    = _bps;    
-#ifdef SOLARIS
+#ifdef UL_SOLARIS
 	if ( ainfo.play.port == AUDIO_SPEAKER )
 		ainfo.play.channels = 1;
 	else
@@ -508,7 +508,7 @@ int slDSP::getDriverBufferSize ()
   
   // HW buffer is 0xffff on my box
 
-#ifdef SOLARIS
+#ifdef UL_SOLARIS
   return ainfo.play.buffer_size;
 #else
   return  1024 * 8;
@@ -528,7 +528,7 @@ void slDSP::getBufferInfo ()
     return ;
   }
     
-#ifndef SOLARIS
+#ifndef UL_SOLARIS
   if( ::ioctl( fd, AUDIO_GETOOFFS, &audio_offset ) < 0)
   {
     perror ( "slDSP: getBufferInfo" ) ;
@@ -557,7 +557,7 @@ void slDSP::write ( void *buffer, size_t length )
 
 float slDSP::secondsRemaining ()
 {
-    return 0.0f ;
+    return 10.0f ;
 }
 
 
@@ -594,10 +594,10 @@ float slDSP::secondsUsed ()
   if ( counter == 0 )
       return 0.0;
 
-#ifndef SOLARIS
-  samples_used = ( counter * init_bytes ) - audio_offset.samples;
-#else
+#ifdef UL_SOLARIS
   samples_used = ( counter * init_bytes ) - ainfo.play.samples;
+#else
+  samples_used = ( counter * init_bytes ) - audio_offset.samples;
 #endif
   
   if (  stereo   ) samples_used /= 2 ;
@@ -609,10 +609,10 @@ float slDSP::secondsUsed ()
 
 void slDSP::sync ()
 { 
-#ifndef SOLARIS
-   if ( !error) ::ioctl ( fd, AUDIO_FLUSH , 0 ) ; 
-#else
+#ifdef UL_SOLARIS
    if ( !error) ::ioctl ( fd, I_FLUSH, FLUSHRW ) ;
+#else
+   if ( !error) ::ioctl ( fd, AUDIO_FLUSH , 0 ) ; 
 #endif
 }
 
@@ -625,47 +625,45 @@ void slDSP::stop ()
 /* SGI IRIX audio                                               */
 /* ------------------------------------------------------------ */
 
-#elif defined(sgi)
+#elif defined(UL_IRIX)
 
-void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
+void slDSP::open ( const char *device, int _rate, int _stereo, int _bps )
 {
-#if 0
   if ( _bps != 8 )
   {
     perror ( "slDSP: supports only 8bit audio for sgi" ) ;
     error = SL_TRUE;
     return;
   }
-#endif
 
-  init_bytes = 1024 * 16;
-  config = alNewConfig();
+  init_bytes = 1024 * 4 ;
+
+  config  = ALnewconfig();
+ 
+  ALsetchannels (  config, _stereo ? AL_STEREO : AL_MONO );
+  ALsetwidth    (  config, _bps == 8 ? AL_SAMPLE_8 : AL_SAMPLE_16 );
+  ALsetqueuesize(  config, init_bytes * 2 );
+
+  port = ALopenport( device, "w", config );
   
-  alSetChannels(config, _stereo ? AL_STEREO : AL_MONO);
-  alSetWidth(config, _bps == 8 ? AL_SAMPLE_8 : AL_SAMPLE_16);
-  alSetQueueSize(config, init_bytes);
-  init_bytes = alGetQueueSize(config);
-
-  port = alOpenPort(device, "w", config);
-    
   if ( port == NULL )
   {
     perror ( "slDSP: open" ) ;
     error = SL_TRUE ;
   }
   else
-  {    
-    ALpv params[1];
+  {
+    long params[2] = {AL_OUTPUT_RATE, 0 };
 
-    params[0].param = AL_OUTPUT_RATE;
-    params[0].value.i = _rate;
-    if ( alSetParams(AL_DEFAULT_OUTPUT, params, 1) != 0 )
-	{
-       perror ( "slDSP: open - alSetParams" ) ;
+    params[1] = _rate;
+
+    if ( ALsetparams(AL_DEFAULT_DEVICE, params, 2) != 0 )
+        {
+       perror ( "slDSP: open - ALsetparams" ) ;
        error = SL_TRUE ;
        return;
-	}
-  
+        }
+ 
     rate    = _rate;
     stereo  = _stereo;
     bps     = _bps;
@@ -680,8 +678,8 @@ void slDSP::close ()
 {
   if ( port != NULL )
   {
-     alClosePort(port);
-     alFreeConfig(config);
+     ALcloseport ( port   );
+     ALfreeconfig( config );
      port = NULL;
   }
 }
@@ -689,16 +687,10 @@ void slDSP::close ()
 
 int slDSP::getDriverBufferSize ()
 {
-  int res; 
-
   if ( error )
     return 0 ;
 
-  res = alGetQueueSize(config);
-  if (  stereo   ) res *= 2 ;
-  if ( bps == 16 ) res *= 2 ;
-
-  return  res;
+  return  ALgetqueuesize( config ) / 2 ;
 }
 
 void slDSP::getBufferInfo ()
@@ -715,14 +707,10 @@ void slDSP::write ( void *buffer, size_t length )
   if ( error || (int)length <= 0 )
     return ;
 
-  // Steve: is this a problem ??
   for ( int i = 0; i < (int)length; i++ )
     buf[i] = buf[i] >> 1;
 
-  if (  stereo   ) length /= 2;
-  if ( bps == 16 ) length /= 2;
-
-  alWriteFrames(port, buffer, length / 2);
+  ALwritesamps(port, (void *)buf, length );
 }
 
 
@@ -733,7 +721,7 @@ float slDSP::secondsRemaining ()
   if ( error )
     return 0.0f ;
 
-  samples_remain = alGetFillable(port);
+  samples_remain = ALgetfillable(port) ;
 
   return   (float) samples_remain / (float) rate ;
 }
@@ -742,11 +730,11 @@ float slDSP::secondsRemaining ()
 float slDSP::secondsUsed ()
 {
   int   samples_used;
-  
+
   if ( error )
     return 0.0f ;
 
-  samples_used = alGetFilled(port);
+  samples_used = ALgetfilled(port) * 2 ;
 
   return   (float) samples_used / (float) rate ;
 }
@@ -772,7 +760,7 @@ void slDSP::stop ()
 #endif
 
 
-#ifdef macintosh
+#if defined(UL_MACINTOSH) || defined(UL_MAC_OSX)
 	
 
 // Print out debugging info when secondsUsed is called,
@@ -832,7 +820,7 @@ void doError( OSErr theError ) {
     msg = "Buffer is too small." ;
     break;
   case paramErr:
-    msg = ("Invalid parameter specified." ;
+    msg = "Invalid parameter specified." ;
     break;
   }
 
@@ -861,7 +849,7 @@ pascal void sndCallbackProc ( SndChannelPtr chan, SndCommand *cmd )
 #endif
 }
 
-void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
+void slDSP::open ( const char *device, int _rate, int _stereo, int _bps )
 {
 
   // Check for valid ranges on inputs
@@ -898,7 +886,7 @@ void slDSP::open ( char *device, int _rate, int _stereo, int _bps )
   //initOptions += initChanRight;    // Right stereo channel
 
   // Define a call-back routine - invoked via callBackCmd
-  SndCallBackUPP callBackRoutine = NewSndCallBackProc ( sndCallbackProc );
+  SndCallBackUPP callBackRoutine = NewSndCallBackUPP ( sndCallbackProc );
 
   // Allocate a sound channel
   sndChannel = new SndChannel;
@@ -1064,7 +1052,7 @@ float slDSP::secondsRemaining ()
     return 0.0f;
   
   // sl doesn't use this, so I didn't write it!
-  return secLeft;
+  return 10.0f;
 }
 
 float slDSP::secondsUsed ()
