@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgTransform.cxx,v 1.9 2001/06/16 16:27:37 stromberg Exp $
+     $Id: ssgTransform.cxx,v 1.13 2002/11/29 19:39:48 sjbaker Exp $
 */
 
 
@@ -110,9 +110,33 @@ void ssgTransform::hot ( sgVec3 s, sgMat4 m, int test_needed )
 
   _ssgPopPath () ;
 
-  postTravTests ( SSGTRAV_HOT ) ; 
+  postTravTests ( SSGTRAV_HOT ) ;
 }
 
+void ssgTransform::los ( sgVec3 s, sgMat4 m, int test_needed )
+{
+  if ( ! preTravTests ( &test_needed, SSGTRAV_LOS ) )
+    return ;
+
+  int los_result = los_test ( s, m, test_needed ) ;
+
+  if ( los_result == SSG_OUTSIDE )
+    return ;
+
+  sgMat4 tmp ;
+
+  sgCopyMat4 ( tmp, m ) ;
+  sgPreMultMat4 ( tmp, transform ) ;
+
+  _ssgPushPath ( this ) ;
+
+  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid() )
+    e -> los ( s, tmp, los_result != SSG_INSIDE ) ;
+
+  _ssgPopPath () ;
+
+  postTravTests ( SSGTRAV_LOS ) ;
+}
 
 void ssgTransform::isect ( sgSphere *s, sgMat4 m, int test_needed )
 {
@@ -175,6 +199,32 @@ void ssgTransform::setTransform ( sgMat4 xform )
 }
 
 
+void ssgTransform::getNetTransform ( sgMat4 xform )
+{
+  if ( getNumParents () > 0 )
+  {
+    getParent ( 0 ) -> getNetTransform ( xform ) ;
+    sgPreMultMat4 ( xform, transform ) ;
+  }
+  else 
+    sgCopyMat4 ( xform, transform ) ;
+}
+
+
+void ssgTransform::getLastNetTransform ( sgMat4 xform )
+{
+  sgMat4 last_xform ;
+  getLastTransform ( last_xform ) ;
+
+  if ( getNumParents () > 0 )
+  {
+    getParent ( 0 ) -> getLastNetTransform ( xform ) ;
+    sgPreMultMat4 ( xform, last_xform ) ;
+  }
+  else 
+    sgCopyMat4 ( xform, last_xform ) ;
+}
+
 
 int ssgTransform::load ( FILE *fd )
 {
